@@ -74,14 +74,14 @@ struct _GSRWindowPrivate {
 	int n_channels, bitrate, samplerate;
 	gboolean has_file;
 	gboolean dirty;
+	gboolean seek_in_progress;
 
 	guint32 tick_id; /* tick_callback timeout ID */
+	guint32 record_id; /* record idle callback timeout ID */
 };
 
 static BonoboWindowClass *parent_class = NULL;
 
-static char *temppath = NULL;
-
 static void
 shutdown_pipeline (GSRWindowPipeline *pipe)
 {
@@ -110,11 +110,21 @@ finalize (GObject *object)
 		g_source_remove (priv->tick_id);
 	}
 
-	shutdown_pipeline (priv->play);
-	g_free (priv->play);
+	if (priv->record_id > 0) {
+		g_source_remove (priv->record_id);
+	}
 
-	shutdown_pipeline (priv->record);
-	g_free (priv->record);
+	g_idle_remove_by_data (window);
+
+	if (priv->play != NULL) {
+		shutdown_pipeline (priv->play);
+		g_free (priv->play);
+	}
+
+	if (priv->record != NULL) {
+		shutdown_pipeline (priv->record);
+		g_free (priv->record);
+	}
 
 	unlink (priv->record_filename);
 	g_free (priv->record_filename);
@@ -755,7 +765,8 @@ fill_in_information (GSRWindow *window,
 	}
 	gtk_label_set_text (GTK_LABEL (fp->filename), text);
 	g_free (text);
-
+	g_free (name);
+	
 	/* Size */
 	if (stat (window->priv->working_file, &buf) == 0) {
 		char *human;
@@ -844,6 +855,7 @@ file_properties (BonoboUIComponent *uic,
 	dialog = gtk_dialog_new_with_buttons (title, GTK_WINDOW (window),
 					      GTK_DIALOG_DESTROY_WITH_PARENT,
 					      GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE, NULL);
+	g_free (title);
 	gtk_window_set_resizable (GTK_WINDOW(dialog), FALSE);
 	fp = g_new (struct _file_props, 1);
 	fp->dialog = dialog;
@@ -987,16 +999,22 @@ help_about (BonoboUIComponent *uic,
 	    const char *path)
 {
 	static GtkWidget *about = NULL;
+	GdkPixbuf *pixbuf = NULL;
 	const char *authors[2] = {"Iain Holmes <iain@prettypeople.org>", NULL};
 
 	if (about != NULL) {
 		gdk_window_raise (about->window);
 		gdk_window_show (about->window);
 	} else {
+		pixbuf = gdk_pixbuf_new_from_file (GNOME_ICONDIR "/gnome-grecord.png", NULL);
 		about = gnome_about_new (_("Sound Recorder"), VERSION,
 					 "Copyright \xc2\xa9 2002 Iain Holmes",
 					 _("A sound recorder for GNOME"),
-					 authors, NULL, NULL, NULL);
+					 authors, NULL, NULL, pixbuf);
+
+		if (pixbuf != NULL)
+			gdk_pixbuf_unref (pixbuf);
+
 		g_signal_connect (G_OBJECT (about), "destroy",
 				  G_CALLBACK (gtk_widget_destroyed), &about);
 		gtk_window_set_transient_for (GTK_WINDOW (about), GTK_WINDOW (window));
@@ -1013,10 +1031,18 @@ media_play (BonoboUIComponent *uic,
 		return;
 	}
 
+	if (window->priv->record == NULL) {
+		return;
+	}
+
 	if (gst_element_get_state (window->priv->record->pipeline) == GST_STATE_PLAYING) {
 		gst_element_set_state (window->priv->record->pipeline, GST_STATE_READY);
 	}
 
+	if (window->priv->play == NULL) {
+		return;
+	}
+
 	gst_element_set_state (window->priv->play->pipeline, GST_STATE_PLAYING);
 }
 
@@ -1050,6 +1076,10 @@ media_record (BonoboUIComponent *uic,
 	      GSRWindow *window,
 	      const char *path)
 {
+	if (window->priv->record == NULL) {
+		return;
+	}
+
 	gst_element_set_state (window->priv->record->pipeline, GST_STATE_PLAYING);
 }
 
@@ -1129,8 +1159,16 @@ get_length (GSRWindow *window)
 	return (gst_element_get_state (window->priv->play->pipeline) == GST_STATE_PLAYING);
 }
 
-static void
-seek_to (GtkRange *range,
+static gboolean
+seek_started (GtkRange *range,GdkEventButton *event,
+	      GSRWindow *window) {
+	g_return_val_if_fail (window->priv != NULL, FALSE);
+	window->priv->seek_in_progress = TRUE;
+	return FALSE;
+}
+
+static gboolean
+seek_to (GtkRange *range,GdkEventButton *gdkevent,
 	 GSRWindow *window)
 {
 	double value = range->adjustment->value;
@@ -1146,9 +1184,12 @@ seek_to (GtkRange *range,
 
 	gst_element_set_state (window->priv->play->pipeline, GST_STATE_PAUSED);
 	time = ((value / 100) * window->priv->len_secs) * GST_SECOND;
+
 	event = gst_event_new_seek (GST_FORMAT_TIME | GST_SEEK_FLAG_FLUSH, time);
 	ret = gst_element_send_event (window->priv->play->sink, event);
 	gst_element_set_state (window->priv->play->pipeline, old_state);
+	window->priv->seek_in_progress = FALSE;
+	return FALSE;
 }
 
 static gboolean
@@ -1173,6 +1214,9 @@ tick_callback (GSRWindow *window)
 		}
 	}
 
+	if (window->priv->seek_in_progress)
+		return TRUE;
+
 	query_worked = gst_element_query (window->priv->play->sink,
 					  GST_QUERY_POSITION, 
 					  &format, &value);
@@ -1181,25 +1225,54 @@ tick_callback (GSRWindow *window)
 		secs = value / GST_SECOND;
 		
 		percentage = ((double) secs / (double) window->priv->len_secs) * 100.0;
-		g_signal_handlers_block_matched (G_OBJECT (window->priv->scale),
-						 G_SIGNAL_MATCH_FUNC,
-						 0, 0, NULL,
-						 G_CALLBACK (seek_to), NULL);
 		gtk_adjustment_set_value (GTK_RANGE (window->priv->scale)->adjustment, percentage + 0.5);
-		g_signal_handlers_unblock_matched (G_OBJECT (window->priv->scale),
-						   G_SIGNAL_MATCH_FUNC,
-						   0, 0, NULL,
-						   G_CALLBACK (seek_to), NULL);
 
 	}
 
 	return (gst_element_get_state (window->priv->play->pipeline) == GST_STATE_PLAYING);
 }
 
+static gboolean
+record_tick_callback (GSRWindow *window)
+{
+	int secs;
+	gint64 value;
+	gboolean query_worked = FALSE;
+	GstFormat format = GST_FORMAT_TIME;
+
+	if (gst_element_get_state (window->priv->record->pipeline) != GST_STATE_PLAYING) {
+		/* This check stops us from doing an unnecessary query */
+		return FALSE;
+	}
+
+	if (window->priv->seek_in_progress)
+		return TRUE;
+
+	query_worked = gst_element_query (window->priv->record->sink,
+					  GST_QUERY_POSITION, 
+					  &format, &value);
+	if (query_worked) {
+		gchar* len_str;
+		secs = value / GST_SECOND;
+		
+		len_str = seconds_to_full_string (secs);
+		window->priv->len_secs = secs;
+		gtk_label_set (GTK_LABEL (window->priv->length), len_str);
+		g_free (len_str);
+	}
+
+	return (gst_element_get_state (window->priv->record->pipeline) == GST_STATE_PLAYING);
+}
+
 static gboolean
 play_iterate (GSRWindow *window)
 {
-	return gst_bin_iterate (GST_BIN (window->priv->play->pipeline));
+	gboolean ret;
+	ret =  gst_bin_iterate (GST_BIN (window->priv->play->pipeline));
+
+	if (!ret)
+		gst_element_set_state (window->priv->play->pipeline, GST_STATE_NULL);
+	return ret;
 }
 
 static void
@@ -1238,8 +1311,10 @@ play_state_changed (GstElement *element,
 		gtk_widget_set_sensitive (window->priv->scale, TRUE);
 		break;
 
-	case GST_STATE_PAUSED:
 	case GST_STATE_READY:
+		gtk_adjustment_set_value (GTK_RANGE (window->priv->scale)->adjustment, 0.0);
+		gtk_widget_set_sensitive (window->priv->scale, FALSE);
+	case GST_STATE_PAUSED:
 		bonobo_ui_component_set_prop (window->priv->ui_component,
 					      "/commands/MediaStop", 
 					      "sensitive", "0", NULL);
@@ -1257,8 +1332,6 @@ play_state_changed (GstElement *element,
 					      "sensitive", "1", NULL);
 		bonobo_ui_component_set_status (window->priv->ui_component,
 						_("Ready"), NULL);
-		gtk_adjustment_set_value (GTK_RANGE (window->priv->scale)->adjustment, 0.0);
-		gtk_widget_set_sensitive (window->priv->scale, FALSE);
 		break;
 	default:
 		break;
@@ -1297,6 +1370,21 @@ play_deep_notify (GstElement *element,
 	}
 }
 
+void
+error_handler (GObject *object, GstObject *orig, gchar *error, GtkWidget *window)
+{
+	GtkWidget *dialog;
+
+	dialog = gtk_message_dialog_new (GTK_WINDOW(window),
+					 GTK_DIALOG_DESTROY_WITH_PARENT,
+					 GTK_MESSAGE_ERROR,
+					 GTK_BUTTONS_CLOSE,
+					 error);
+	gtk_dialog_run (GTK_DIALOG (dialog));
+	gtk_widget_destroy (dialog);
+}
+
+
 static GSRWindowPipeline *
 make_play_pipeline (GSRWindow *window)
 {
@@ -1311,6 +1399,9 @@ make_play_pipeline (GSRWindow *window)
 	g_signal_connect (G_OBJECT (pipeline->pipeline), "deep-notify",
 			  G_CALLBACK (play_deep_notify), window);
 
+	g_signal_connect (G_OBJECT (pipeline->pipeline), "error",
+			  G_CALLBACK (error_handler), window);
+
 	pipeline->src = gst_element_factory_make ("filesrc", "src");
  	spider = gst_element_factory_make ("spider", "spider");
 	pipeline->sink = gst_element_factory_make ("osssink", "sink");
@@ -1325,44 +1416,39 @@ make_play_pipeline (GSRWindow *window)
 	return pipeline;
 }
 
-static void
-record_state_changed (GstElement *element,
-		      GstElementState old,
-		      GstElementState state,
-		      GSRWindow *window)
+static gboolean
+record_start(gpointer user_data) 
 {
-	
-	switch (state) {
-	case GST_STATE_PLAYING:
-#if 0 /* Don't need this yet...but we will eventually I think */
-		if (window->priv->len_secs == 0) {
-			window->priv->get_length_attempts = 16;
-			g_timeout_add (200, (GSourceFunc) get_length, window);
-			g_timeout_add (200, (GSourceFunc) tick_callback, window);
-		}
-#endif
-		bonobo_ui_component_set_prop (window->priv->ui_component,
-					      "/commands/MediaStop", 
-					      "sensitive", "1", NULL);
-		bonobo_ui_component_set_prop (window->priv->ui_component,
-					      "/commands/MediaPlay", 
-					      "sensitive", "0", NULL);
-		bonobo_ui_component_set_prop (window->priv->ui_component,
-					      "/commands/MediaRecord", 
-					      "sensitive", "0", NULL);
-		bonobo_ui_component_set_prop (window->priv->ui_component,
-					      "/commands/FileSave",
-					      "sensitive", "0", NULL);
-		bonobo_ui_component_set_prop (window->priv->ui_component,
-					      "/commands/FileSaveAs",
-					      "sensitive", "0", NULL);
-		bonobo_ui_component_set_status (window->priv->ui_component,
-						_("Recording..."), NULL);
-		gtk_widget_set_sensitive (window->priv->scale, FALSE);
-		break;
+	GSRWindow *window = GSR_WINDOW (user_data);
+	if (window->priv->len_secs == 0) {
+		window->priv->get_length_attempts = 16;
+		g_timeout_add (200, (GSourceFunc) record_tick_callback, window);
+	}
+	bonobo_ui_component_set_prop (window->priv->ui_component,
+				      "/commands/MediaStop", 
+				      "sensitive", "1", NULL);
+	bonobo_ui_component_set_prop (window->priv->ui_component,
+				      "/commands/MediaPlay", 
+				      "sensitive", "0", NULL);
+	bonobo_ui_component_set_prop (window->priv->ui_component,
+				      "/commands/MediaRecord", 
+				      "sensitive", "0", NULL);
+	bonobo_ui_component_set_prop (window->priv->ui_component,
+				      "/commands/FileSave",
+				      "sensitive", "0", NULL);
+	bonobo_ui_component_set_prop (window->priv->ui_component,
+				      "/commands/FileSaveAs",
+				      "sensitive", "0", NULL);
+	bonobo_ui_component_set_status (window->priv->ui_component,
+					_("Recording..."), NULL);
+	gtk_widget_set_sensitive (window->priv->scale, FALSE);
+	window->priv->record_id = 0;
+	return FALSE;
+}
 
-	case GST_STATE_PAUSED:
-	case GST_STATE_READY:
+static gboolean
+record_ready_or_pause(gpointer user_data) {
+	GSRWindow *window = GSR_WINDOW (user_data);
 		bonobo_ui_component_set_prop (window->priv->ui_component,
 					      "/commands/MediaStop", 
 					      "sensitive", "0", NULL);
@@ -1381,6 +1467,26 @@ record_state_changed (GstElement *element,
 		bonobo_ui_component_set_status (window->priv->ui_component,
 						_("Ready"), NULL);
 		gtk_widget_set_sensitive (window->priv->scale, FALSE);
+		window->priv->record_id = 0;
+		return FALSE;
+}
+
+static void
+record_state_changed (GstElement *element,
+		      GstElementState old,
+		      GstElementState state,
+		      GSRWindow *window)
+{
+	switch (state) {
+	case GST_STATE_PLAYING:
+		window->priv->record_id = g_idle_add (record_start, window);
+		break;
+
+	case GST_STATE_READY:
+		gtk_adjustment_set_value (GTK_RANGE (window->priv->scale)->adjustment, 0.0);
+		gtk_widget_set_sensitive (window->priv->scale, FALSE);
+	case GST_STATE_PAUSED:
+		window->priv->record_id = g_idle_add (record_ready_or_pause, window);
 		break;
 	default:
 		break;
@@ -1398,12 +1504,15 @@ make_record_pipeline (GSRWindow *window)
 						      "state-change",
 						      G_CALLBACK (record_state_changed),
 						      window);
+	g_signal_connect (G_OBJECT (pipeline->pipeline), "error",
+			  G_CALLBACK (error_handler), window);
+
 	pipeline->src = gst_element_factory_make ("osssrc", "src");
 	encoder = gst_element_factory_make ("wavenc", "encoder");
 	if (encoder == NULL) {
 		g_warning ("You do not have a new enough gst-plugins.\n"
 			   "You really need CVS after 20-Oct-2002");
-		return;
+		return NULL;
 	}
 	pipeline->sink = gst_element_factory_make ("filesink", "sink");
 
@@ -1422,10 +1531,6 @@ init (GSRWindow *window)
 {
 	GSRWindowPrivate *priv;
 
-	if (temppath == NULL) {
-		temppath = g_strdup ("/tmp/");
-	}
-
 	window->priv = g_new0 (GSRWindowPrivate, 1);
 	priv = window->priv;
 
@@ -1465,6 +1570,8 @@ init (GSRWindow *window)
 	priv->len_secs = 0;
 	priv->get_length_attempts = 16;
 	priv->dirty = FALSE;
+
+	return;
 }
 
 GType
@@ -1494,8 +1601,13 @@ calculate_format_value (GtkScale *scale,
 {
 	int seconds;
 
-	seconds = window->priv->len_secs * (value / 100);
-	return seconds_to_string (seconds);
+	if (gst_element_get_state (window->priv->record->pipeline) == GST_STATE_PLAYING) {
+		seconds = value;
+		return seconds_to_string (seconds);
+	} else {
+		seconds = window->priv->len_secs * (value / 100);
+		return seconds_to_string (seconds);
+	}
 }
 	
 GtkWidget *
@@ -1528,7 +1640,7 @@ gsr_window_new (const char *filename)
 	}
 
 	window->priv->record_filename = g_strdup_printf ("%s/gsr-record-%s-%d.XXXXXX",
-							 temppath, filename, getpid ());
+							 g_get_tmp_dir(), filename, getpid ());
 	window->priv->record_fd = mkstemp (window->priv->record_filename);
 	close (window->priv->record_fd);
 
@@ -1540,9 +1652,11 @@ gsr_window_new (const char *filename)
 		window->priv->working_file = g_strdup (filename);
 	}
 
-	g_object_set (G_OBJECT (window->priv->record->sink),
-		      "location", window->priv->record_filename,
-		      NULL);
+	if (window->priv->record != NULL) {
+		g_object_set (G_OBJECT (window->priv->record->sink),
+			      "location", window->priv->record_filename,
+			      NULL);
+	}
 
 	gtk_window_set_default_size (GTK_WINDOW (window), 512, 200);
 	hbox = gtk_hbox_new (FALSE, 6);
@@ -1550,9 +1664,12 @@ gsr_window_new (const char *filename)
 	gtk_box_pack_start (GTK_BOX (window->priv->main_vbox), hbox, FALSE, FALSE, 0);
 	
 	window->priv->scale = gtk_hscale_new (GTK_ADJUSTMENT (gtk_adjustment_new (0, 0, 100, 1, 1, 0)));
+	window->priv->seek_in_progress = FALSE;
 	g_signal_connect (G_OBJECT (window->priv->scale), "format-value",
 			  G_CALLBACK (calculate_format_value), window);
-	g_signal_connect (G_OBJECT (window->priv->scale), "value-changed",
+	g_signal_connect (G_OBJECT (window->priv->scale), "button_press_event",
+			  G_CALLBACK (seek_started), window);
+	g_signal_connect (G_OBJECT (window->priv->scale), "button_release_event",
 			  G_CALLBACK (seek_to), window);
 
 	gtk_scale_set_value_pos (GTK_SCALE (window->priv->scale), GTK_POS_BOTTOM);
